(document:surround "/std/frame")

(document:root
	(when loaded
		(form-bind "document:expert_mode" "change" change-mode)
		(change-mode)
		(update-roles "")
		(change-role)
		(update-system-roles "")
		(change-system-role)
	)
)

(define (change-mode)
	(if (eq? 1 (form-cookie "expert_mode"))
		(begin
			(form-update-visibility "expert_mode" #t)
			(form-update-visibility "simple_mode" #f)
			(update-roles "")
			(change-role)
		)
		(begin
			(form-update-visibility "expert_mode" #f)
			(form-update-visibility "simple_mode" #t)
			(update-system-roles "")
			(change-system-role)
		)
	)
)

(define (update-roles value)
	(let ((data (woo-list "/role/get_roles")))
		(form-update-enum "roles" data)
		(form-update-value "roles" value)
		(if (string= (form-value "roles") "")
			(form-update-value "roles" (woo-get-option (cond-car data) 'name)))
	)
)

(define (update-system-roles value)
	(let ((data (woo-list "/role/get_system_roles")))
		(form-update-enum "system_roles" data)
		(form-update-value "system_roles" value)
		(if (string= (form-value "system_roles") "")
			(form-update-value "system_roles" (woo-get-option (cond-car data) 'name)))
	)
)

(define (change-role)
	(let* ((role (form-value "roles"))
		(privs (woo-list "/role/get_privs_role" 'role role)))
		(form-update-enum "privs_role" privs)
	)
	(let* ((role (form-value "roles"))
		(privs (woo-list "/role/get_privs_role_d" 'role role)))
		(form-update-enum "privs_role_d" privs)
	)
)

(define (change-system-role)
	(let* ((role (form-value "system_roles"))
		(privs (woo-list "/role/get_privs_for_system_role_from_its_file" 'role role)))
		(form-update-enum "privs_system_role" privs)
	)
	(let* ((role (form-value "system_roles"))
		(privs (woo-list "/role/get_privs_for_system_role_from_other" 'role role)))
		(form-update-enum "privs_system_other" privs)
	)
)

(define (add-role)
	(form-popup "/role/roleadd" 'role (form-value "roles"))
	(update-roles (form-value "roles"))
	(change-role)
)

(define (del-role)
	(woo-catch/message
		(thunk
			(woo-write "/role/del_role"
				'role (form-value "roles")
				'privilege (form-value "privs_role"))
		)
	)
	(update-roles (form-value "roles"))
	(change-role)
)

(define (add-role-d)
	(form-popup "/role/roleadd_d" 'role (form-value "roles"))
	(update-roles (form-value "roles"))
	(change-role)
)

(define (del-role-d)
	(define word (form-value "privs_role_d"))
	(if (string? word)
		(if (>= (string-length word) 10)
			(begin
				(let
					(
						(filename (cadr (string-split word #\()))
						(priv (car (string-split word #\()))
					)

					(woo-catch/message
						(thunk
							(woo-write "/role/del_role_d"
								'file (string-pad-right filename (- (string-length filename) 1))
								'role (form-value "roles")
								'privilege (string-pad-right priv (- (string-length priv) 1)))
						)
					)
					(update-roles (form-value "roles"))
					(change-role)
				)
			)
		)
	)
)

(define (add-system-role)
	(form-popup "/role/roleadd_sys" 'role (form-value "system_roles"))
	(update-system-roles (form-value "system_roles"))
	(change-system-role)
)

(define (del-system-role)
	(woo-catch/message
		(thunk
			(woo-write "/role/del_system_role"
				'role (form-value "system_roles")
				'privilege (form-value "privs_system_role"))
		)
	)
	(update-system-roles (form-value "system_roles"))
	(change-system-role)
)

(vbox name "expert_mode"
	margin 20

	(hbox
		(document:id roles (listbox name "roles"
									header (vector (_ "Roles"))
									enumref "/role/get_roles"
									(when changed (change-role))
							)
		)

		(document:id privs_role (listbox name "privs_role" header (vector (_ "Privileges from /etc/role"))))
		(document:id privs_role_d (listbox name "privs_role_d" header (vector (_ "Privileges from /etc/role.d/*"))))
	)

	(hbox
		(spacer)

		(hbox align "right"
			(button (_ "Add a privilege...") name "add_role_button"
				(when clicked (add-role))
			)
			(button (_ "Remove a privilege") name "remove_role_button"
				(when clicked (del-role))
			)
		)

		(hbox align "right"
			(button (_ "Add a privilege...") name "add_priv_button"
				(when clicked (add-role-d))
			)
			(button (_ "Remove a privilege") name "remove_from_roled_button"
				(when clicked (del-role-d))
			)
		)
	)
)

(vbox name "simple_mode"
	margin 20

	(hbox
		(document:id system_roles (listbox name "system_roles"
									header (vector (_ "System roles"))
									enumref "/role/get_system_roles"
									(when changed (change-system-role))
								)
		)

		(vbox
			(document:id privs_system_role (listbox name "privs_system_role" header (vector (_ "System role privileges"))))
			(document:id privs_system_other (listbox name "privs_system_other" activity #f
				header (vector (_ "Other privileges"))))
		)
	)

	(hbox
		(spacer)

		(hbox align "right"
			(button (_ "Add a privilege...") name "add_system_role_button"
				(when clicked (add-system-role))
			)
			(button (_ "Remove a privilege") name "remove_system_role_button"
				(when clicked (del-system-role))
			)
		)
	)
)
